==> 2.0.0-beta10&file=out_yasapi-nt-2.0.0-beta10-src.tar.gz: out_yasapi-nt-2.0.0-beta10/gyasapi_config.h <==
/*
 * yasapi_config.h
 *
 * Yet Another (WA)SAPI (YASAPI) NT Output Plugin for Winamp (out_yasapi-nt)
 * Copyright (C) 2018  Peter Belkner (pbelkner@snafu.de)
 *
 * out_yasapi-nt is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * out_yasapi-nt is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef __YASAPI_CONFIG_H__ // [
#define __YASAPI_CONFIG_H__
#include <yasapi_param.h>

#ifdef __cplusplus // [
extern "C" {
#endif // ]
///////////////////////////////////////////////////////////////////////////////
#define NOTEBOOK(members) \
  { &gcIClassNotebookVtbl,1,0,members }
#define PAGE(rows) \
  { &gcIClassPageVtbl,1,0,rows }
#define FRAME(rows) \
  { &gcIClassFrameVtbl,1,0,rows }
#define BUTTON_BOX() \
  { &gcIClassButtonBoxVtbl }

#if defined (YASAPI_DEBUG) // [
////////////////
#define CELL_TT(lpVtbl,nSpan,bCenter,data,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  NULL, \
  -1, \
  NULL, \
  0, \
  tt, \
}
#define CELL(lpVtbl,nSpan,bCenter,data,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  #widget, \
  offsetof(Config,widget), \
  NULL, \
  0, \
  tt, \
}
#define CELL_COMMON(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  #widget, \
  offsetof(Config,widget), \
  NavigateOptionsCommonConfig, \
  offsetof(OptionsCommon,field), \
  tt, \
}
#define CELL_PLAYER(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  #widget, \
  offsetof(Config,widget), \
  NavigatePlayer, \
  offsetof(Player,field), \
  tt, \
}
#define CELL_DEVICE(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  #widget, \
  offsetof(Config,widget), \
  NavigateOptionsDeviceConfig, \
  offsetof(OptionsDevice,field), \
  tt, \
}
#define CELL_ENGINE(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  #widget, \
  offsetof(Config,widget), \
  NavigateEngine, \
  offsetof(Engine,field), \
  tt, \
}
#define CELL_CONFIG(lpVtbl,nSpan,bCenter,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  #widget, \
  offsetof(Config,widget), \
  NULL, \
  -1, \
  tt, \
}
#define CELL_MATRIX(lpVtbl,nSpan,bCenter,data,widget,field,row,col) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  #widget "[" #row "][" #col "]", \
  offsetof(Config,widget[row][col]), \
  NULL, \
  offsetof(FormatMatrix,field[row][col]) \
}
#else // ] [
////////////////
#define CELL_TT(lpVtbl,nSpan,bCenter,data,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  -1, \
  NULL, \
  0, \
  tt, \
}
#define CELL(lpVtbl,nSpan,bCenter,data,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NULL, \
  0, \
  tt, \
}
#define CELL_COMMON(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateOptionsCommonConfig, \
  offsetof(OptionsCommon,field), \
  tt, \
}
#define CELL_PLAYER(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigatePlayer, \
  offsetof(Player,field), \
  tt, \
}
#define CELL_DEVICE(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateOptionsDeviceConfig, \
  offsetof(OptionsDevice,field), \
  tt, \
}
#define CELL_ENGINE(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateEngine, \
  offsetof(Engine,field), \
  tt, \
}
#define CELL_CONFIG(lpVtbl,nSpan,bCenter,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  offsetof(Config,widget), \
  NULL, \
  -1, \
  tt, \
}
#define CELL_MATRIX(lpVtbl,nSpan,bCenter,data,widget,field,row,col) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  offsetof(Config,widget[row][col]), \
  NULL, \
  offsetof(FormatMatrix,field[row][col]) \
}
#endif // ]
#define LABEL_FRAMES_ENGINE(nSpan,bCenter,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelEngineFramesVtbl,nSpan,bCenter,NULL,widget,\
      field,tt)
#if 0 // [
#define LABEL_ENGINE_LATENCY(nSpan,bCenter,widget) \
  CELL(&gcIClassLabelEngineLatencyVtbl,nSpan,bCenter,NULL,widget)
#endif // ]
#define LABEL_PLAYER_LATENCY(nSpan,bCenter,widget) \
  CELL(&gcIClassLabelPlayerLatencyVtbl,nSpan,bCenter,NULL,widget,NULL)
#if 0 // [
#define LABEL_STATIC(nSpan,bCenter,label) \
  { &gcIClassLabelStaticVtbl,nSpan,bCenter,label }
#endif // ]
#define LABEL_STATIC(nSpan,bCenter,label,tt) \
  CELL_TT(&gcIClassLabelStaticVtbl,nSpan,bCenter,label,tt)
#define LABEL_STATIC_FUN(nSpan,bCenter,fun) \
  { &gcIClassLabelStaticFunVtbl,nSpan,bCenter,fun }
#define LABEL_UINT32_PLAYER(nSpan,bCenter,format,widget,field,tt) \
  CELL_PLAYER(&gcIClassLabelUint32Vtbl,nSpan,bCenter,format,widget,field,tt)
#define LABEL_UINT32_ENGINE(nSpan,bCenter,format,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelUint32Vtbl,nSpan,bCenter,format,widget,field,tt)
#define LABEL_ENGINE(nSpan,bCenter,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelVtbl,nSpan,bCenter,NULL,widget,field,tt)
#define COMBO_BOX_COMMON(nSpan,bCenter,widget,field,map,tt) \
  CELL_COMMON(&gcIClassComboBoxVtbl,nSpan,bCenter,map,widget,field,tt)
#define COMBO_BOX_DEVICE(nSpan,bCenter,widget,field,map,tt) \
  CELL_DEVICE(&gcIClassComboBoxVtbl,nSpan,bCenter,map,widget,field,tt)
#define COMBO_BOX_DEVICE_WASAPI(nSpan,bCenter,widget,tt) \
  CELL_CONFIG(&gcIClassComboBoxDeviceVtbl,nSpan,bCenter,widget,tt)
#define CHECK_BUTTON_COMMON(nSpan,bCenter,widget,field,tt) \
  CELL_COMMON(&gcIClassCheckButtonVtbl,nSpan,bCenter,NULL,widget,field,tt)
#define CHECK_BUTTON_DEVICE(nSpan,bCenter,widget,field,tt) \
  CELL_DEVICE(&gcIClassCheckButtonVtbl,nSpan,bCenter,NULL,widget,field,tt)
#define LABEL_MAPPED_DEVICE(nSpan,bCenter,widget,field,map,tt) \
  CELL_DEVICE(&gcIClassLabelMappedVtbl,nSpan,bCenter,map,widget,field,tt)
#define LABEL_MAPPED_COMMON(nSpan,bCenter,widget,field,map) \
  CELL_COMMON(&gcIClassLabelMappedVtbl,nSpan,bCenter,map,widget,field,NULL)
#define LABEL_DEVICE_WASAPI(nSpan,bCenter,widget) \
  CELL_CONFIG(&gcIClassLabelDeviceVtbl,nSpan,bCenter,widget,NULL)
#define SCALE_DEVICE(nSpan,bCenter,widget,field,param,tt) \
  CELL_DEVICE(&gcIClassScaleVtbl,nSpan,bCenter,param,widget,field,tt)
#define LEVEL_BAR_ENGINE(nSpan,bCenter,data,widget,field,tt) \
  CELL_ENGINE(&gcIClassLevelBarVtbl,nSpan,bCenter,data,widget,field,tt)
#define LABEL_MATRIX_BITS(nSpan,bCenter,col) { \
  &gcIClassLabelMatrixBitsVtbl, \
  nSpan, \
  bCenter, \
  (void *)(int)(col), \
}
#define LABEL_MATRIX_FREQ(nSpan,bCenter,row) { \
  &gcIClassLabelMatrixFreqVtbl, \
  nSpan, \
  bCenter, \
  (void *)(int)(row), \
}
#define CHECK_BUTTON_MATRIX(nSpan,bCenter,widget,field,row,col) \
  CELL_MATRIX(&gcIClassCheckButtonMatrixVtbl,nSpan,bCenter,data,widget,\
      field,row,col)

///////////////////////////////////////////////////////////////////////////////
extern const char CONFIG_WIDGET[];

///////////////////////////////////////////////////////////////////////////////
typedef const struct _ClassNotebookPage ClassNotebookPage;
typedef const struct _ConfigComboBoxMap ConfigComboBoxMap;
typedef const struct _ConfigScaleParameter ConfigScaleParameter;
typedef const struct _ConfigLevelBarParameter ConfigLevelBarParameter;
typedef enum _EPanel _EPanel;
typedef const struct _IClassCellVtbl IClassCellVtbl;
typedef struct _Expand Expand;

///////////////////////////////////////////////////////////////////////////////
struct _ConfigComboBoxMap {
  const char *label;
  const char *label_em;
  int value;
};

struct _ConfigScaleParameter {
#if defined (YASAPI_PLAYBACK_BEGIN_MIN_NUM_CHUNKS) // [
  union {
    int nRefOffs;

    struct {
#endif // ]
      double min;
      double max;
      double step;
#if defined (YASAPI_PLAYBACK_BEGIN_MIN_NUM_CHUNKS) // [
    };
  };

  int bRef;
#endif // ]
};

struct _ConfigLevelBarParameter {
  int dummy;
};

///////////////////////////////////////////////////////////////////////////////
enum _EPanel {
  ePanelSpace=4,
};

///////////////////////////////////////////////////////////////////////////////
struct _Expand {
  int h,v;
};

///////////////////////////////////////////////////////////////////////////////
struct _IClassCellVtbl {
#if defined (YASAPI_DEBUG) // [
  const char *pId;
#endif // ]
  GtkWidget *(*create)(ClassCell *pCell, Dialog *pDialog, Expand *pExpand);
};

struct _ClassCell {
  IClassCellVtbl *lpVtbl;
  int nSpan;
  int bCenter;
  const void *data;
#if defined (YASAPI_DEBUG) // [
  const char *pId;
#endif // ]
  int widget;
  char *(*nav)(Dialog *pDialog, int offs);
  int offs;
  const char *tooltip;
};

typedef enum _ClassRowLimits ClassRowLimits;

enum _ClassRowLimits {
  eClassRowNumCols=16,
};

typedef ClassCell ClassRow[eClassRowNumCols];

typedef ClassRow ClassPage[];

struct _ClassNotebookPage {
  ClassCell label;
  ClassCell page;
};

typedef ClassNotebookPage ClassNotebook[];

///////////////////////////////////////////////////////////////////////////////
extern IClassCellVtbl gcIClassLabelEngineFramesVtbl;
//extern IClassCellVtbl gcIClassLabelEngineLatencyVtbl;
extern IClassCellVtbl gcIClassLabelPlayerLatencyVtbl;
extern IClassCellVtbl gcIClassLabelVtbl;
extern IClassCellVtbl gcIClassLabelUint32Vtbl;
extern IClassCellVtbl gcIClassNotebookVtbl;
extern IClassCellVtbl gcIClassPageVtbl;
extern IClassCellVtbl gcIClassFrameVtbl;
extern IClassCellVtbl gcIClassButtonBoxVtbl;
extern IClassCellVtbl gcIClassLabelStaticVtbl;
extern IClassCellVtbl gcIClassComboBoxVtbl;
extern IClassCellVtbl gcIClassCheckButtonVtbl;
extern IClassCellVtbl gcIClassLabelDeviceVtbl;
extern IClassCellVtbl gcIClassComboBoxDeviceVtbl;
extern IClassCellVtbl gcIClassLabelMappedVtbl;
extern IClassCellVtbl gcIClassScaleVtbl;
extern IClassCellVtbl gcIClassLabelStaticFunVtbl;
extern IClassCellVtbl gcIClassLevelBarVtbl;
extern IClassCellVtbl gcIClassLabelMatrixBitsVtbl;
extern IClassCellVtbl gcIClassLabelMatrixFreqVtbl;
extern IClassCellVtbl gcIClassCheckButtonMatrixVtbl;

///////////////////////////////////////////////////////////////////////////////
ConfigWidget *ConfigWidgetCreate(ClassCell *pCell, Dialog *pDialog,
    IConfigWidgetVtbl *lpVtbl, GtkWidget *widget);

char *NavigatePlayer(Dialog *pDialog, int offs);
char *NavigateDeviceSelectedConfig(Dialog *pDialog, int offs);
char *NavigateOptionsCommonConfig(Dialog *pDialog, int offs);
char *NavigateOptionsDeviceConfig(Dialog *pDialog, int offs);
char *NavigateEngine(Dialog *pDialog, int offs);

#ifdef __cplusplus // [
}
#endif // ]
#endif // __YASAPI_CONFIG_H__ ]

==> 2.3.0-beta10&file=out_yasapi-nt-2.3.0-beta10-src.tar.gz: out_yasapi-nt-2.3.0-beta10/gyasapi_config.h <==
/*
 * yasapi_config.h
 *
 * Yet Another (WA)SAPI (YASAPI) New Technology (NT) Output (out_yasapi-nt)
 * Copyright (C) 2015-2019  Peter Belkner (pbelkner@snafu.de)
 *
 * out_yasapi-nt is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * out_yasapi-nt is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with out_yasapi-nt.  If not, see <https://www.gnu.org/licenses/>.
 */
#ifndef __YASAPI_CONFIG_H__ // [
#define __YASAPI_CONFIG_H__
#include <yasapi_param.h>

#ifdef __cplusplus // [
extern "C" {
#endif // ]

///////////////////////////////////////////////////////////////////////////////
#define NOTEBOOK_CONFIG(members,gtk_widget) { \
  .lpVtbl=&gcIClassNotebookVtbl, \
  .nSpan=1, \
  .bCenter=0, \
  .data=members, \
  .widget=offsetof(Config,gtk_widget), \
  .nav=NavigateConfig, \
}
#define PAGE(rows) \
  { &gcIClassPageVtbl,1,0,rows }
#define FRAME(nSpan,rows) \
  { &gcIClassFrameVtbl,nSpan,0,rows }
#define BUTTON_BOX() \
  { &gcIClassButtonBoxVtbl }

////////////////
#define CELL_TT(lpVtbl,nSpan,bCenter,data,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  -1, \
  NULL, \
  0, \
  tt, \
}
#define CELL(lpVtbl,nSpan,bCenter,data,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NULL, \
  0, \
  tt, \
}
#define CELL_COMMON(lpVtbl,nSpan,bCenter,data,widget,field,se,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateOptionsCommonConfig, \
  offsetof(OptionsCommon,field), \
  tt, \
  se, \
}
#define CELL_PLAYER(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigatePlayer, \
  offsetof(Player,field), \
  tt, \
}
#define CELL_DEVICE(lpVtbl,nSpan,bCenter,data,widget,field,se,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateOptionsDeviceConfig, \
  offsetof(OptionsDevice,field), \
  tt, \
  se, \
}
#define CELL_MONITOR(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateEngineClientMonitor, \
  offsetof(EngineClientMonitor,field), \
  tt, \
}
#define CELL_ENGINE(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateEngineClient, \
  offsetof(EngineClient,field), \
  tt, \
}
#define CELL_ENGINE_SERVER(lpVtbl,nSpan,bCenter,data,widget,field,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  data, \
  offsetof(Config,widget), \
  NavigateEngineServer, \
  offsetof(EngineServer,field), \
  tt, \
}
#define CELL_CONFIG(lpVtbl,nSpan,bCenter,widget,tt) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  offsetof(Config,widget), \
  NULL, \
  -1, \
  tt, \
}
#define CELL_MATRIX(lpVtbl,nSpan,bCenter,data,widget,field,row,col) { \
  lpVtbl, \
  nSpan, \
  bCenter, \
  NULL, \
  offsetof(Config,widget[row][col]), \
  NULL, \
  offsetof(FormatMatrix,field[row][col]) \
}

#define LABEL_FRAMES_ENGINE(nSpan,bCenter,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelEngineClientFramesVtbl,nSpan,bCenter,NULL,widget,\
      field,tt)
#define LABEL_PLAYER_LATENCY(nSpan,bCenter,widget) \
  CELL(&gcIClassLabelPlayerLatencyVtbl,nSpan,bCenter,NULL,widget,NULL)
#define LABEL_STATIC(nSpan,bCenter,label,tt) \
  CELL_TT(&gcIClassLabelStaticVtbl,nSpan,bCenter,label,tt)
#define LABEL_STATIC_FUN(nSpan,bCenter,fun) \
  { &gcIClassLabelStaticFunVtbl,nSpan,bCenter,fun }
#define LABEL_UINT32_PLAYER(nSpan,bCenter,format,widget,field,tt) \
  CELL_PLAYER(&gcIClassLabelUint32Vtbl,nSpan,bCenter,format,widget,field,tt)
#define LABEL_DOUBLE_PLAYER(nSpan,bCenter,format,widget,field,tt) \
  CELL_PLAYER(&gcIClassLabelDoubleVtbl,nSpan,bCenter,format,widget,field,tt)
#define LABEL_UINT32_ENGINE(nSpan,bCenter,format,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelUint32Vtbl,nSpan,bCenter,format,widget,field,tt)
#define LABEL_ENGINE(nSpan,bCenter,widget,field,tt) \
  CELL_ENGINE(&gcIClassLabelVtbl,nSpan,bCenter,NULL,widget,field,tt)
#define LABEL_ENGINE_SERVER(nSpan,bCenter,widget,field,tt) \
  CELL_ENGINE_SERVER(&gcIClassLabelVtbl,nSpan,bCenter,NULL,widget,field,tt)
#define COMBO_BOX_COMMON(nSpan,bCenter,widget,field,map,se,tt) \
  CELL_COMMON(&gcIClassComboBoxVtbl,nSpan,bCenter,map,widget,field,se,tt)
#define COMBO_BOX_DEVICE(nSpan,bCenter,widget,field,map,se,tt) \
  CELL_DEVICE(&gcIClassComboBoxVtbl,nSpan,bCenter,map,widget,field,se,tt)
#define COMBO_BOX_DEVICE_WASAPI(nSpan,bCenter,widget,tt) \
  CELL_CONFIG(&gcIClassComboBoxDeviceVtbl,nSpan,bCenter,widget,tt)
#define CHECK_BUTTON_COMMON_SE(nSpan,bCenter,widget,field,se,tt) \
  CELL_COMMON(&gcIClassCheckButtonVtbl,nSpan,bCenter,NULL,widget,field,se,tt)
#define CHECK_BUTTON_COMMON(nSpan,bCenter,widget,field,tt) \
  CELL_COMMON(&gcIClassCheckButtonVtbl,nSpan,bCenter,NULL,widget,field,NULL,tt)
#define CHECK_BUTTON_DEVICE(nSpan,bCenter,widget,field,tt) \
  CELL_DEVICE(&gcIClassCheckButtonVtbl,nSpan,bCenter,NULL,widget,field,NULL,tt)
#define LABEL_MAPPED_DEVICE(nSpan,bCenter,widget,field,map,tt) \
  CELL_DEVICE(&gcIClassLabelMappedVtbl,nSpan,bCenter,map,widget,field,NULL,tt)
#define LABEL_MAPPED_COMMON(nSpan,bCenter,widget,field,map) \
  CELL_COMMON(&gcIClassLabelMappedVtbl,nSpan,bCenter,map,widget,field,NULL)
#define LABEL_DEVICE_WASAPI(nSpan,bCenter,widget) \
  CELL_CONFIG(&gcIClassLabelDeviceVtbl,nSpan,bCenter,widget,NULL)
#define SCALE_COMMON(nSpan,bCenter,widget,field,param,tt) \
  CELL_COMMON(&gcIClassScaleVtbl,nSpan,bCenter,param,widget,field,NULL,tt)
#define SCALE_DEVICE(nSpan,bCenter,widget,field,param,tt) \
  CELL_DEVICE(&gcIClassScaleVtbl,nSpan,bCenter,param,widget,field,NULL,tt)
#define SCALE_MONITOR(nSpan,bCenter,widget,field,param,tt) \
  CELL_MONITOR(&gcIClassScaleVtbl,nSpan,bCenter,param,widget,field,tt)
#define LEVEL_BAR_ENGINE(nSpan,bCenter,data,widget,field,tt) \
  CELL_ENGINE(&gcIClassProgessBarVtbl,nSpan,bCenter,data,widget,field,tt)
#define LABEL_MATRIX_BITS(nSpan,bCenter,col) { \
  &gcIClassLabelMatrixBitsVtbl, \
  nSpan, \
  bCenter, \
  (void *)(int)(col), \
}
#define LABEL_MATRIX_FREQ(nSpan,bCenter,row) { \
  &gcIClassLabelMatrixFreqVtbl, \
  nSpan, \
  bCenter, \
  (void *)(int)(row), \
}
#define CHECK_BUTTON_MATRIX(nSpan,bCenter,widget,field,row,col) \
  CELL_MATRIX(&gcIClassCheckButtonMatrixVtbl,nSpan,bCenter,data,widget,\
      field,row,col)
#define LABEL_VOLUME(nSpan,bCenter,widget,tt) \
  CELL(&gcIClassLabelVolumeVtbl,nSpan,bCenter,NULL,widget,tt)

///////////////////////////////////////////////////////////////////////////////
extern const char CONFIG_WIDGET[];

///////////////////////////////////////////////////////////////////////////////
typedef const struct _ClassNotebookPage ClassNotebookPage;
typedef const struct _ConfigComboBoxMap ConfigComboBoxMap;
typedef const struct _ConfigScaleParameter ConfigScaleParameter;
typedef const struct _ConfigColorTheme ConfigColorTheme;
typedef const struct _ConfigProgessBarParameter ConfigProgessBarParameter;
typedef enum _EPanel _EPanel;
typedef const struct _IClassCellVtbl IClassCellVtbl;
typedef struct _Expand Expand;

///////////////////////////////////////////////////////////////////////////////
struct _ConfigComboBoxMap {
  const char *label;
  const char *label_em;
  int value;
};

struct _ConfigScaleParameter {
  double min;
  double max;
  double step;
};

struct _ConfigColorTheme {
  struct {
    int red;
    int green;
    int blue;
  } fill;

  struct {
    int red;
    int green;
    int blue;
  } border;
};
;

struct _ConfigProgessBarParameter {
  int height;
};

///////////////////////////////////////////////////////////////////////////////
enum _EPanel {
  ePanelSpace=4,
};

///////////////////////////////////////////////////////////////////////////////
struct _Expand {
  int h,v;
};

///////////////////////////////////////////////////////////////////////////////
struct _IClassCellVtbl {
  GtkWidget *(*create)(ClassCell *pCell, Dialog *pDialog, Expand *pExpand);
};

struct _ClassCell {
  IClassCellVtbl *lpVtbl;
  int nSpan;
  int bCenter;
  const void *data;
  int widget;
  char *(*nav)(Dialog *pDialog, int offs);
  int offs;
  const char *tooltip;
  void (*SideEffect)(ConfigWidget *pWidget);
};

typedef enum _ClassRowLimits ClassRowLimits;

enum _ClassRowLimits {
  eClassRowNumCols=16,
};

typedef ClassCell ClassRow[eClassRowNumCols];

typedef ClassRow ClassPage[];

struct _ClassNotebookPage {
  ClassCell label;
  ClassCell page;
};

typedef ClassNotebookPage ClassNotebook[];

///////////////////////////////////////////////////////////////////////////////
extern IClassCellVtbl gcIClassLabelEngineClientFramesVtbl;
extern IClassCellVtbl gcIClassLabelPlayerLatencyVtbl;
extern IClassCellVtbl gcIClassLabelVtbl;
extern IClassCellVtbl gcIClassLabelDoubleVtbl;
extern IClassCellVtbl gcIClassLabelUint32Vtbl;
extern IClassCellVtbl gcIClassNotebookVtbl;
extern IClassCellVtbl gcIClassPageVtbl;
extern IClassCellVtbl gcIClassFrameVtbl;
extern IClassCellVtbl gcIClassButtonBoxVtbl;
extern IClassCellVtbl gcIClassLabelStaticVtbl;
extern IClassCellVtbl gcIClassComboBoxVtbl;
extern IClassCellVtbl gcIClassCheckButtonVtbl;
extern IClassCellVtbl gcIClassLabelDeviceVtbl;
extern IClassCellVtbl gcIClassComboBoxDeviceVtbl;
extern IClassCellVtbl gcIClassLabelMappedVtbl;
extern IClassCellVtbl gcIClassScaleVtbl;
extern IClassCellVtbl gcIClassLabelStaticFunVtbl;
extern IClassCellVtbl gcIClassProgessBarVtbl;
extern IClassCellVtbl gcIClassLabelMatrixBitsVtbl;
extern IClassCellVtbl gcIClassLabelMatrixFreqVtbl;
extern IClassCellVtbl gcIClassCheckButtonMatrixVtbl;
extern IClassCellVtbl gcIClassLabelVolumeVtbl;

void ConfigMonitorEngineClientTailSetStyle(ConfigWidget *pWidget);
void ConfigMonitorPaddingSetStyle(ConfigWidget *pWidget);
void ConfigMonitorStartSetStyle(ConfigWidget *pWidget);
void ConfigMonitorWrittenSetStyle(ConfigWidget *pWidget);


///////////////////////////////////////////////////////////////////////////////
ConfigWidget *ConfigWidgetCreate(ClassCell *pCell, Dialog *pDialog,
    IConfigWidgetVtbl *lpVtbl, GtkWidget *widget);

char *NavigatePlayer(Dialog *pDialog, int offs);
char *NavigateDeviceSelectedConfig(Dialog *pDialog, int offs);
char *NavigateOptionsCommonConfig(Dialog *pDialog, int offs);
char *NavigateOptionsDeviceConfig(Dialog *pDialog, int offs);
char *NavigateEngineClient(Dialog *pDialog, int offs);
char *NavigateEngineServer(Dialog *pDialog, int offs);
char *NavigateEngineClientMonitor(Dialog *pDialog, int offs);
char *NavigateConfig(Dialog *pDialog, int offs);

#ifdef __cplusplus // [
}
#endif // ]
#endif // __YASAPI_CONFIG_H__ ]
